#![warn(clippy::all, clippy::pedantic)]
#![doc = include_str!("../README.md")]

mod decode;
mod encode;
mod error;

pub use {decode::Decoder, encode::Encoder, error::Error as B64Error};

#[derive(Clone, Copy)]
/// Defines the character set used for encoding and decoding
pub struct B64Alphabet {
    items: [u8; 64],
    pad: u8,
}

/// The most common Base64 alphabet as defined in [RFC4648](https://www.rfc-editor.org/rfc/rfc4648)
pub static B64_RFC4648_ALPHABET: B64Alphabet = B64Alphabet {
    items: [
        b'A', b'B', b'C', b'D', b'E', b'F', b'G', b'H', b'I', b'J', b'K', b'L', b'M', b'N', b'O',
        b'P', b'Q', b'R', b'S', b'T', b'U', b'V', b'W', b'X', b'Y', b'Z', b'a', b'b', b'c', b'd',
        b'e', b'f', b'g', b'h', b'i', b'j', b'k', b'l', b'm', b'n', b'o', b'p', b'q', b'r', b's',
        b't', b'u', b'v', b'w', b'x', b'y', b'z', b'0', b'1', b'2', b'3', b'4', b'5', b'6', b'7',
        b'8', b'9', b'+', b'/',
    ],
    pad: b'=',
};

impl Default for B64Alphabet {
    fn default() -> Self {
        B64_RFC4648_ALPHABET
    }
}

impl B64Alphabet {
    pub(crate) fn get(&self, c: u8) -> Option<usize> {
        for (idx, x) in self.items.iter().enumerate() {
            if *x == c {
                return Some(idx);
            }
        }
        None
    }

    pub(crate) fn pad(&self) -> u8 {
        self.pad
    }
}
